from .qubopt_common import *

MAX_SKM_CHANNEL_COUNT = 128

qo.skmtreeiface.argtypes = (PNode, POINTER(PNode), QTR_Callback, QTR_Callback, QTR_Callback)
qo.skmtreeiface.restype = PNode

def SKM(config, source_segs, on_report=lambda s: None, on_percent=lambda pct: None, main_hold=None, **kw):
    cfg = prep_tree(config)
    for k, v in kw.iteritems():
        cfg[k].data = v
    dat = source_to_dataset(source_segs)
    dat['ChannelCount'].data = cfg['ModelFile']['ChannelCount'].data[0] = \
        min(MAX_SKM_CHANNEL_COUNT, cfg['ModelFile']['ChannelCount'].data[0])
    if main_hold is None:
        def fill_data(seg):
            src = source_segs[0].clone()
            src.f, src.l = seg.data[:2]
            seg['Channel'].data = src.get_samples().samples.astype('float64').flat
            return seg
    else:
        def fill_data(seg):
            with main_hold:
                src = source_segs[0].clone()
                src.f, src.l = seg.data[:2]
                seg['Channel'].data = src.get_samples().samples.astype('float64').reshape((src.l-src.f+1,))
            return seg
    ptr_array = (PNode * 2)()
    ptr_array[0] = dat.impl
    ptr_array[1] = None
    def on_report_node(node):
        on_report(str(node.data))
        return node
    def on_percent_node(node):
        on_percent(node.data[0])
        return node
    result_impl = qo.skmtreeiface(cfg.impl, ptr_array, wrap_callback(fill_data), wrap_callback(on_report_node), wrap_callback(on_percent_node))
    result = qubx.tree_native.Node(impl=result_impl)
    qubx.tree_native.qubtree.QTR_DECREF(result_impl)
    return result

def get_skm_config(model, fixed_amp=[], fixed_sd=[], reestimate=False, max_iterations=10, conv_ll=0.01, fix_q=False,
                   drop_first_last=False, drop_if_class=False, drop_class=0):
    config = qubx.tree_native.Node('Properties')
    config.append( prep_tree(model.as_tree()) )
    Nclass = len(model.classes)
    config["FixAmp"].data = [(i in fixed_amp) and 1 or 0 for i in xrange(Nclass)]
    config["FixSD"].data = [(i in fixed_sd) and 1 or 0 for i in xrange(Nclass)]
    config["Reestimate"].data = int(reestimate)
    config["MaxIterations"].data = max_iterations
    config["ConvLL"].data = conv_ll
    config["FixQ"].data = int(fix_q)
    config["DropFirstLast"].data = int(drop_first_last)
    config["DropIfClass"].data = int(drop_if_class)
    config["DropClass"].data = drop_class
    config["StopFlag"].data = 0
    return config

